//--------------------------------------------------------------------------
//
// Environment:
//      This software is part of models developed at BES collaboration
//      based on the EvtGen framework.  If you use all or part
//      of it, please give an appropriate acknowledgement.
//
// Copyright Information: See EvtGen/BesCopyright
//      Copyright (A) 2006      Ping Rong-Gang @IHEP
//
// Module:  EvtDIY.cc
//
// Description:  Class to boost a daughter momentum into the mother helicity system
//
// Modification history:
//
//    Ping R.-G.       December, 2006       Module created
//
//------------------------------------------------------------------------
// 

//#include "EvtGenBase/EvtPatches.hh"
#include <iostream>
#include <math.h>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <strstream>
#include "EvtHelSys.h"
#include "EvtdFunction.h"
using namespace std;  //::endl;

EvtHelSys::~EvtHelSys() 
{
}

EvtHelSys::EvtHelSys( const TLorentzVector& p4p, const TLorentzVector& p4d) 
{
   _p4parent=p4p;
   _p4daug  =p4d;
}

EvtHelSys::EvtHelSys( ) 
{
}

double  EvtHelSys::getHelAng(const int i) 
{
   TLorentzVector b_p4p,rp4p, rp4d,boostdaug; 
   //   EvtVector3R GetHelAng;
   while (_p4parent.Rho()>=1e-10) 
      {
	 b_p4p=-1 * _p4parent;   //boost from Lab to HEL sys. required to reverse mom.Vec.
	 b_p4p.SetE(_p4parent.E());  
	 _bp4p=b_p4p;

	 // first to rotate the mother and daugher momentum to the helicity system
	 double theta=Angles(_p4parent,1);
	 double phi  =Angles(_p4parent,2);

	 rp4p=Helrotate(_p4parent,phi,theta);
	 rp4d=Helrotate(_p4daug,  phi,theta);
	 // then boos to the CM system
	 TLorentzVector r_p4p = -1*rp4p;  //boost from Lab to HEL sys. required to reverse mom.Vec.
	 r_p4p.SetE(rp4p.E());
	 rp4p.Boost(r_p4p.BoostVector());
	 rp4d.Boost(r_p4p.BoostVector());
	 boostdaug=rp4d;

	 _rotatep4p=rp4p;
	 _rotatep4d=rp4d;
	 _bst=boostdaug;
	 _bp4p=r_p4p;
	 return Angles(_bst,i);  //_bst:daughter momentum in helicity system, i=0==>|_bst|;i=1,2==>(theta,phi)
      }
   return Angles(_p4daug,i);
}

TLorentzVector EvtHelSys::checkparent(){ return _p4parent;}
TLorentzVector EvtHelSys::checkdaug()  { return _p4daug;  }
TLorentzVector EvtHelSys::checkst(int i)
{
   getHelAng(1);
   if(i==0) return _bp4p;      //parent momentum used to boost the daughter to the CM sys.  
   if(i==1) return _rotatep4p; //the parent momentum in Hel system by rotation
   if(i==2) return _rotatep4d; //the daughter momentum in Hel. system by rotation
   if(i==3) return _bst;       //_bst:daughter momentum in helicity system
}

TLorentzVector EvtHelSys::Helrotate(const TLorentzVector p1, const double phi, const double theta)
{
   TLorentzVector Rp;
   double cp=cos(phi);
   double sp=sin(phi);
   double ct=cos(theta);
   double st=sin(theta);
   double t=p1.E(), x=p1.X(), y=p1.Y(),   z=p1.Z();
   double xp=x*cp*ct+y*sp*ct-z*st, yp= -x*sp+y*cp, zp=x*cp*st+y*sp*st+z*ct;
   Rp.SetXYZT(xp,yp,zp,t);
   return Rp;
}

double EvtHelSys::Angles(const TLorentzVector bbst,const int i)
{
   double rxy=sqrt(pow(bbst.X(),2.)+pow(bbst.Y(),2.));
   if(bbst.Rho()<=1e-6) 
      {
	 if(i==0) return 0.;
	 else if (i==1) return 0.;
	 else if (i==2) return 0.;
	 else {cout<<"Angles(i): i<=2"<<endl;abort();}
      }
   else if(rxy<=1e-6)
      {
	 if(i==0) return bbst.Rho();
	 if(i==1) return 0.;
	 if(i==2) return 0.;
	 else {cout<<"Angles(i): i<=2"<<endl;abort();}
      }
   else 
      {
	 double theta=acos(bbst.Z()/bbst.Rho());
	 double csphi=bbst.X()/bbst.Rho()/sin(theta);
	 if(fabs(csphi)>1.0) csphi=csphi/fabs(csphi);
	 double phi=acos(csphi);
	 if(bbst.Y()<0.0) phi=2*3.1415926-phi;
	 if(i==0) return bbst.Rho();
	 else if (i==1) return theta;
	 else if (i==2) return phi;
	 else {cout<<"Angles(i): i<=2"<<endl;abort();}
      }
} 

//double djmn(int j, int m, int n, double theta)
//{  
//   int j2=j*2,m2=m*2,n2=n*2;
//   double temp=EvtdFunction::d(j2,m2,n2,theta);
//   return temp;
//}
//
//double djmn(double j, double m, double n, double theta)
//{  
//   int j2=(int)(j*2*1.1),m2=(int)(m*2*1.1),n2=(int)(n*2*1.1);
//   double temp=EvtdFunction::d(j2,m2,n2,theta);
//   return temp;
//} 
//
//
//TComplex Djmn(int j, int m, int n, double phi, double theta, double gamma)
//{
//   int j2=j*2,m2=m*2,n2=n*2;
//   TComplex gp(cos(-phi*m  ), -sin(phi*m));
//   TComplex gm(cos(-gamma*n), -sin(gamma*n));
//   double  tp3=EvtdFunction::d(j2,m2,n2,theta);
//
//   TComplex temp=gp * tp3 * gm;
//
//   return temp;
//}
//  
//
//TComplex Djmn(double j, double m, double n, double phi,double theta, double gamma)
//{
//   int j2=(int)(j*2*1.1),m2=(int)(m*2*1.1),n2=(int)(n*2*1.1);
//   TComplex gp(cos(-phi*m  ), -sin(phi*m));
//   TComplex gm(cos(-gamma*n), -sin(gamma*n)); 
//   double  tp3=EvtdFunction::d(j2,m2,n2,theta);
//   TComplex  temp=gp * tp3 * gm;
//   return temp;
//}
//
