// C++ include
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <cctype>
#include <string>
//------ ROOT includes ---------
#include <TMath.h>
#include <TFile.h>
#include <TF1.h>
#include <TH1.h>
#include <TH2.h>
#include <TH3.h>
#include <TTree.h>
#include <TNtuple.h>
#include <TBenchmark.h>
#include <TROOT.h>
#include <TKey.h>
#include <TLine.h>
#include <TDirectory.h>
#include <TString.h>
#include <TSystem.h>
#include <TRandom.h>
#include <TCanvas.h>
#include <TStyle.h>

#include <TComplex.h>
//
#include <TLorentzVector.h>
#include <TGenPhaseSpace.h>
#include "EvtdFunctionSingle.h"
#include "EvtdFunction.h"
#include "EvtHelSys.h"

using namespace std;
//
TComplex bw(Double_t m , Double_t  Mass, Double_t Gamma )
{
   return -Mass*Gamma/(m*m-Mass*Mass+TComplex::I()*Mass*Gamma); 
}

double djmn(int j, int m, int n, double theta)
{
   int j2=j*2,m2=m*2,n2=n*2;
   double temp=EvtdFunction::d(j2,m2,n2,theta);
   return temp;
}

double djmn(double j, double m, double n, double theta)
{
   int j2=(int)(j*2*1.1),m2=(int)(m*2*1.1),n2=(int)(n*2*1.1);
   double temp=EvtdFunction::d(j2,m2,n2,theta);
   return temp;
}


TComplex Djmn(int j, int m, int n, double phi, double theta, double gamma)
{
   int j2=j*2,m2=m*2,n2=n*2;
   TComplex gp(cos(-phi*m  ), -sin(phi*m));
   TComplex gm(cos(-gamma*n), -sin(gamma*n));
   double  tp3=EvtdFunction::d(j2,m2,n2,theta);

   TComplex temp=gp * tp3 * gm;

   return temp;
}


TComplex Djmn(double j, double m, double n, double phi,double theta, double gamma)
{
   int j2=(int)(j*2*1.1),m2=(int)(m*2*1.1),n2=(int)(n*2*1.1);
   TComplex gp(cos(-phi*m  ), -sin(phi*m));
   TComplex gm(cos(-gamma*n), -sin(gamma*n));
   double  tp3=EvtdFunction::d(j2,m2,n2,theta);
   TComplex  temp=gp * tp3 * gm;
   return temp;
}



Double_t rhopi( const TLorentzVector *pPip, const TLorentzVector *pPim,  const TLorentzVector *pPiz )
{
   TLorentzVector pPipm=*pPip+*pPim;
   TLorentzVector pPimz=*pPim+*pPiz;
   TLorentzVector pPizp=*pPiz+*pPip;
   TLorentzVector W=*pPip+*pPim+*pPiz;
   //
   Double_t mrho0=1.02000, Gamma0=0.0100;
   Double_t mrho1=1.45000, Gamma1=0.4000;
   Double_t mrho2=2.15000, Gamma2=0.4000;
   //
   EvtHelSys anglepm(pPipm,*pPip);
   EvtHelSys anglemz(pPimz,*pPim);
   EvtHelSys anglezp(pPizp,*pPiz);
   //
   Double_t  thetapm = anglepm.getHelAng(1);
   Double_t  thetamz = anglemz.getHelAng(1);
   Double_t  thetazp = anglezp.getHelAng(1);
   //
   Double_t  phipm   = anglepm.getHelAng(2);
   Double_t  phimz   = anglemz.getHelAng(2);
   Double_t  phizp   = anglezp.getHelAng(2);
   //
   EvtHelSys anglez(W,pPipm);
   EvtHelSys anglem(W,pPimz);
   EvtHelSys anglep(W,pPizp);
   Double_t  thetaz  = anglez.getHelAng(1);
   Double_t  thetam  = anglem.getHelAng(1);
   Double_t  thetap  = anglep.getHelAng(1);
   Double_t    phiz  = anglez.getHelAng(2);
   Double_t    phim  = anglem.getHelAng(2);
   Double_t    phip  = anglep.getHelAng(2);
   //
   Double_t  dgamma  = 0;
   Double_t  gamma   = 0;
   Double_t  rho0   = 1, rho1=0.0, rho2=0.0;
   for (Int_t i=-1;i<=1;i=i+2)
      {
	 TComplex amprhoz=0.;
	 TComplex amprhom=0.;
	 TComplex amprhop=0.;
	 for (Int_t j=-1;j<=1;j=j+2)
	    {
	       TComplex AmpAnglepm = (TComplex)j * Djmn(1,i,j, phiz,thetaz,gamma) * Djmn(1,j,0, phipm,thetapm,gamma);
	       TComplex AmpAnglemz = (TComplex)j * Djmn(1,i,j, phim,thetam,gamma) * Djmn(1,j,0, phimz,thetamz,gamma);  
	       TComplex AmpAnglezp = (TComplex)j * Djmn(1,i,j, phip,thetap,gamma) * Djmn(1,j,0, phizp,thetazp,gamma);
	       //
	       amprhoz = amprhoz+AmpAnglepm*
		  (bw(pPipm.M(),mrho0,Gamma0)*rho0 + 
		   bw(pPipm.M(),mrho1,Gamma1)*rho1 + 
		   bw(pPipm.M(),mrho2,Gamma2)*rho2  );
	       /*
	       amprhom = amprhom+AmpAnglemz*
		  (bw(pPimz.M(),mrho0,Gamma0)*rho0 + 
		   bw(pPimz.M(),mrho1,Gamma1)*rho1 + 
		   bw(pPimz.M(),mrho2,Gamma2)*rho2  ); 
	       amprhop = amprhop+AmpAnglezp*
		  (bw(pPizp.M(),mrho0,Gamma0)*rho0 + 
		   bw(pPizp.M(),mrho1,Gamma1)*rho1 + 
		   bw(pPizp.M(),mrho2,Gamma2)*rho2  );
	       */
	    }
	 dgamma=dgamma+(amprhoz+amprhop+amprhom)*TComplex::Conjugate(amprhoz+amprhop+amprhom);
      }
   //
   return dgamma;
}
//
int main() {
   // psi->3pi,  using of TGenPhaseSpace (Momentum, Energy units are Gev/C, GeV)
   // if (!gROOT->GetClass("TGenPhaseSpace")) gSystem.Load("libPhysics");
   //
   gRandom->SetSeed(20090316);
   TFile *f  = new TFile("rhopi.root","RECREATE");
   TH2F *h2  = new TH2F("h2 ","", 140,-1.,13., 140,-1.,13.);
   TH1F *rp  = new TH1F("rp ","", 200, 0.,4.);
   TH1F *rm  = new TH1F("rm ","", 200, 0.,4.);
   TH1F *rz  = new TH1F("rz ","", 200, 0.,4.);
   TH1F *cs0 = new TH1F("cs0","", 100,-1.,1.);
   TH1F *cs1 = new TH1F("cs1","", 100,-1.,1.);
   //
   TLorentzVector eplus (0.0, 0.0, 1.843, 1.843);
   TLorentzVector eminus(0.0, 0.0,-1.843, 1.843);
   TLorentzVector W = eplus + eminus;
   Double_t masses[3] = { 0.493, 0.493, 0.547} ;
   
   const Int_t nevent=1000;
   
   TGenPhaseSpace event;
   event.SetDecay(W, 3, masses);
   Double_t MaxWt = 0.;
   Double_t MaxRh = 0.;
   for (Int_t n=0;n<nevent;n++)
      {
	 Double_t weight = event.Generate();
	 TLorentzVector *pPip    = event.GetDecay(0);
	 TLorentzVector *pPim    = event.GetDecay(1);
	 TLorentzVector *pPiz    = event.GetDecay(2);
	 Double_t wate_rho       = rhopi( pPip, pPim,  pPiz );
	 if ( weight   > MaxWt ) MaxWt=weight*1.05;
	 if ( wate_rho > MaxRh ) MaxRh=wate_rho*1.05;
      }
   std::cout<< "Max Wate phsp = "<<MaxWt<<std::endl;
   std::cout<< "Max Wate rhop = "<<MaxRh<<std::endl;
   //
   Int_t i,j,n;
   for (n=0,i=0,j=0;j<nevent;i++) 
      {
	 Double_t weight = event.Generate();
	 if (weight> MaxWt)
	    {
	       std::cout<<"Max : "<<MaxWt<<" --> "<<weight*1.05<<std::endl;
	       MaxWt = weight*1.05; 
	    } 
	 if( weight < gRandom->Rndm()*MaxWt) continue;
	 n++ ; 
	 TLorentzVector *pPip    = event.GetDecay(0);
	 TLorentzVector *pPim    = event.GetDecay(1);
	 TLorentzVector *pPiz    = event.GetDecay(2);
	 Double_t wate_rho=rhopi( pPip, pPim,  pPiz);
	 //
	 if ( wate_rho > MaxRh) 
	    {
	       std::cout<<"Max : "<<MaxRh<<" --> "<<wate_rho*1.05<<std::endl;
	       MaxRh = wate_rho*1.05;
	    } 
	 if ( wate_rho < gRandom->Rndm()*MaxRh) continue;
	 j++ ; 
	 //
	 TLorentzVector pPPiz = *pPip + *pPim;
	 TLorentzVector pPPim = *pPim + *pPiz;
	 TLorentzVector pPPip = *pPiz + *pPip;
	 EvtHelSys angle0(W,pPPiz);
	 EvtHelSys angle1(pPPiz,*pPip);
	 //
	 Double_t  theta0=angle0.getHelAng(1);
	 Double_t  theta1=angle1.getHelAng(1);
	 //	 
	 h2 ->Fill(pPPip.M2() , pPPim.M2(), 1.0   );
	 rp ->Fill(pPPiz.M()  , 1.0   );
	 rm ->Fill(pPPim.M()  , 1.0   );
	 rz ->Fill(pPPiz.M()  , 1.0   );
	 cs0->Fill(cos(theta0), 1.0   );
	 cs1->Fill(cos(theta1), 1.0   );
      }
   //
   std::cout<<"totol try times = "<<i<<std::endl;
   std::cout<<"totol gen  PHSP = "<<n<<std::endl;
   std::cout<<"totol gen rhopi = "<<j<<std::endl;
   TCanvas *c1 =new TCanvas("rhopi","E of gamma",900,1000) ;
   c1->Divide(2,2);
   c1->cd(1);    h2 ->Draw();
   c1->cd(2);    rp ->Draw();
   c1->cd(3);    cs0->Draw();
   c1->cd(4);    cs1->Draw();
   c1->Print("rhopi.eps");
   f->Write();
   return 0;
}

